#!/usr/bin/env python3
DESCRIPTION = """
    This script helps with the installation and updating of the PADME PHT station software.
    Note that for (re)starting the software, it may need root privileges depending on your docker installation.
    To stop and start the software manually, use the docker-compose commands with the CWD as the folder where the docker-compose is stored.
    docker-compose start
    docker-compose stop 

    To change installation location set the environment variable PADME to the path where the software should be started.
    Note that the PADME variable has to be set the same everytime you invoke this script.
"""

import subprocess
import argparse
import os


GIT_LOCATION = "https://github.com/LaurenzNeumann/TestInstallPHT.git"

CONNECTION_ERROR_MESSAGE = "Error while connecting to docker daemon. Check above error of compose.\n Docker sometimes need root priviliges."
INSTALL_HELP_MESSAGE = "Install the software by downloading the docker compose file"
UPDATE_HELP_MESSAGE = "Update the Software, by checking for a new docker compose file and pulling images."

remove_trailing_slash = lambda x: x[:-1] if x[-1]=="/" else x

get_home_location = lambda: os.environ["HOME"]  if "HOME" in os.environ else "./"
INSTALLATION_LOCATION = os.environ["PADME"] if "PADME" in os.environ else f"{remove_trailing_slash(get_home_location())}/.PADME"

def check_for_wget() -> bool:
    cp = subprocess.run(["which", "wget"])
    return cp.returncode == 0

def check_for_curl() -> bool:
    cp = subprocess.run(["which", "curl"])
    return cp.returncode == 0 

def check_for_git() -> bool:
    cp = subprocess.run(["which", "git"], capture_output=True)
    return cp.returncode == 0

def check_for_dockercomposer() -> bool:
    cp = subprocess.run(["which", "docker-compose"], capture_output=True)
    return cp.returncode == 0 

def up(path: str):
    cp = subprocess.run(["docker-compose", "up", "-d"], cwd=f"{remove_trailing_slash(path)}")
    if cp.returncode != 0:
        print("Something went wrong while (re)starting the software!")
        exit(1)
    return True

def compose_pull(path: str):
    cp = subprocess.run(["docker-compose", "pull"], cwd=f"{remove_trailing_slash(path)}")
    if cp.returncode != 0:
        print("Something went wrong while pulling docker images!")
        exit(1)
    return True

def update():
    # inspired by update script of oh-my-zsh
    # https://github.com/ohmyzsh/ohmyzsh/blob/master/tools/upgrade.sh
    # set git autostash so that changes to the local files are stashed and applied after pull
    cp = subprocess.run(["git", "config", "rebase.autoStash", "true"])
    # get last commit
    cp = subprocess.run(["git", "rev-parse", "HEAD"], cwd=INSTALLATION_LOCATION, capture_output=True)
    last_commit = str(cp.stdout)

    #try to pull new version
    cp = subprocess.run(["git", "pull", "--rebase", "--stat", "origin", "main"], check=True, cwd=INSTALLATION_LOCATION)

    # check if HEAD has a different commit now
    cp = subprocess.run(["git", "rev-parse", "HEAD"], cwd=INSTALLATION_LOCATION, capture_output=True)
    if str(cp.stdout) == last_commit:
        print("Files did not change.")
    # currently we do not build the images, therefore 
    # it is possible that while the files are unchanged, that the images are updated
    # therefore we do a pull with docker-compose nevertheless
    print("Update images...")
    compose_pull(INSTALLATION_LOCATION)

    print("Update complete.")
    start_software = input("Do you want to (re)start the software? [y/N]:")
    if start_software:
        up(INSTALLATION_LOCATION)
    print("Finished.")

def install():
    print("Installing PADME PHT Station Software...")
    print(f"Software will be installed in {INSTALLATION_LOCATION}")

    if input("Continue? [Y/n]") == "n": exit(0)

    print("Clone PADME Repo...")

    cp = subprocess.run(["git", "clone", GIT_LOCATION, INSTALLATION_LOCATION])
    if cp.returncode != 0:
        print("Error while cloning repo!")
        exit(1)
    # in future: build images here

    start_software = input("Do you want to start the software? [y/N]:")
    if start_software.lower() == "y":
        if up(INSTALLATION_LOCATION):
            print("Station software started. Visit localhost:8080 to finish the installation.")

    print("Wizard finished")

def check_for_dependencies():
    if not check_for_dockercomposer():
        print("No docker-compose command found! Please install docker-compose and make sure it is executable from this context!")
        exit(1)

    if not check_for_git():
        print("git not found. Please install docker-compose and make sure it is executable from this context!")
        exit(1)
    
def main(): 
    parser = argparse.ArgumentParser(description="Installation and update script for the Padme PHT architecture.")
    parser.description = DESCRIPTION
    parser.add_argument("--install", action="store_true")
    parser.add_argument("--update", action="store_true")
    args = parser.parse_args()
    check_for_dependencies()
    if not (args.install ^ args.update):
        print("Please specify one command.")
    if args.install:
        install()
    elif args.update:
        update()
main()